/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSGTKMMHOSTENTRY_H
#define CONEXUSGTKMMHOSTENTRY_H

#include <string>
#include <gtkmm.h>
#include <conexus.h>
#include <conexus-gtkmm/enums.h>

namespace Conexus
{
  namespace Gtk
  {

    /**
     *
     * TODO provide custom hostname lookups, kinda like DNS but for images...
     * @author Rick L Vinyard Jr
     */
    class HostEntry : public ::Gtk::VBox
    {
      public:
        HostEntry( unsigned lookup = LOOKUP_IPV4|LOOKUP_IPV6 );
        HostEntry( IPAddressInterface* addr, unsigned lookup = LOOKUP_IPV4|LOOKUP_IPV6 );
        HostEntry( IPAddressInterface::pointer addr, unsigned lookup = LOOKUP_IPV4|LOOKUP_IPV6 );
        HostEntry( IPv4::Address& address, unsigned lookup = LOOKUP_IPV4|LOOKUP_IPV6 );
        HostEntry( IPv6::Address& address, unsigned lookup = LOOKUP_IPV4|LOOKUP_IPV6 );

        ~HostEntry();

        IPAddressInterface* address();

        void set_address( IPAddressInterface* addr, unsigned lookup = LOOKUP_IPV4 );
        void set_address( IPAddressInterface::pointer addr, unsigned lookup = LOOKUP_IPV4 );
        void set_address( IPv4::Address& addr, unsigned lookup = LOOKUP_IPV4 );
        void set_address( IPv6::Address& addr, unsigned lookup = LOOKUP_IPV6 );

        void unset_address();

        ::Gtk::ComboBoxEntryText& host_widget();
        ::Gtk::Image& good_host_image();
        ::Gtk::Image& bad_host_image();
        std::string host();
        void set_host( const std::string& s );

        bool is_hostname_label_visible();
        bool is_host_label_visible();
        bool is_host_image_visible();

        void show_hostname_label ( bool show=true );
        void show_host_label ( bool show=true );
        void show_host_image ( bool show=true );

        ::Gtk::Button& apply_button();

        bool is_immediate_apply_enabled();

        void enable_immediate_apply( bool enable=true );

        void apply();

      protected:
        IPAddressInterface* m_address;
        
        unsigned m_lookup;
        bool m_immediate_apply;
        
        sigc::connection m_conn_address_host;

        sigc::connection m_host_widget_connection;

        ::Gtk::Label m_hostname_label;
        ::Gtk::Label m_host_label;
        ::Gtk::ComboBoxEntryText m_host_widget;
        ::Gtk::EventBox m_image_holder;
        ::Gtk::Image m_good_host_image;
        ::Gtk::Image m_bad_host_image;
        ::Gtk::Button m_apply_button;

        virtual void on_controlled_host_changed();
        
        virtual void on_host_widget_changed();

      private:
        void setup_widget();

    };

  }

}

#endif
